from selenium.webdriver.chrome.options import Options
from selenium import webdriver
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.common.by import By
from selenium.webdriver.support import expected_conditions
from selenium.common.exceptions import NoSuchElementException
from PIL import Image, ImageTk

import sys

import chromedriver_autoinstaller as autoInstall
autoInstall.install()

chromeOptions = Options()
chromeOptions.add_argument("--headless")
chromeOptions.add_experimental_option("excludeSwitches", ["enable-logging"])
chromeOptions.add_argument("window-size=1920, 1080")

browser = webdriver.Chrome("chromedriver.exe" if sys.platform.startswith("win") else "chromedriver", options=chromeOptions)

def waitToBeClickable(byType, path):
    return WebDriverWait(browser, 10).until(expected_conditions.element_to_be_clickable((getattr(By, byType), path)))

class User:

    def __init__(self, loginObject):
        assert type(loginObject) == dict, "Expected dictonary got " + str(type(loginObject))
        assert loginObject["Username"] and loginObject["Password"], "Expected username and password in dictonary"

        username, password = loginObject["Username"], loginObject["Password"]

        try:
            browser.get("https://auth.ioeducation.com/users/sign_in")
        except:
            return print("\033[91mCould not connect to pupilpath\033[0m"), quit()

        data = [
            ("//*[@id='user_username']", username),
            ("//*[@id='user_password']", password)
        ]

        for typePath, data in data:
            waitToBeClickable("XPATH", typePath).send_keys(data)

            waitToBeClickable("ID", "sign_in").click()

        browser.get("https://pupilpath.skedula.com/home/dashboard/")

        waitToBeClickable("ID", "sign_in").click()

        try:
            browser.find_element_by_xpath('//*[@id="messages"]/div')
        except NoSuchElementException:
            if browser.current_url == "https://auth.ioeducation.com/users/sign_in":
                return print("\033[91mUsername/Password is incorrect\033[0m"), quit()

        waitToBeClickable("XPATH", '//*[@id="loginSKD"]/span[2]').click()

        self.Root = browser

        self.Name = browser.find_element_by_xpath('/html/body/div[1]/div[1]/div/div/a[2]').text.split(": ")[1]
        self.Classes = len(browser.find_elements_by_xpath("//*[@id='progress-card']/tbody/tr"))
        
    def GetClassData(self):
        studentData = {"Title": [], "Teacher": [], "Department": [], "Average": []}

        for i in (number + 1 for number in range(self.Classes)):
            elements = {
                "Title": browser.find_element_by_xpath(f'//*[@id="progress-card"]/tbody/tr[{i}]/td[2]').text,

                "Teacher": browser.find_element_by_xpath(f'//*[@id="progress-card"]/tbody/tr[{i}]/td[3]').text,

                "Department": browser.find_element_by_xpath(f'//*[@id="progress-card"]/tbody/tr[{i}]/td[4]').text,
                
                "Average": float(browser.find_element_by_xpath(f'//*[@id="progress-card"]/tbody/tr[{i}]/td[5]/span').text) if browser.find_elements_by_xpath(f'//*[@id="progress-card"]/tbody/tr[{i}]/td[5]/span') else "-"
            }

            for element in elements:
                fetchedElement = elements.get(element)

                studentData[element].append(fetchedElement.strip() if type(fetchedElement) == str else fetchedElement)

        return studentData

    def GetGradeData(self):
        filteredList = list(filter(lambda i: isinstance(i, float), self.GetClassData()["Average"]))
        studentAverage = sum(filteredList) / len(filteredList)

        def getOverallGrade(grade):
            gradesMapping = {0: "Failing", 60: "Borderline", 80: "Average", 90: "Honors"}

            key = None

            for mappedGrade in gradesMapping.keys():
                if mappedGrade <= grade:
                    key = mappedGrade
            
            return gradesMapping[key]

        return {"Average" : studentAverage, "Overall" : getOverallGrade(studentAverage)}

    def SaveGrades(self, path):
        assert type(path) == str, "Expected string while saving grades got " + str(type(path))

        self.Root.save_screenshot(path)

        openedScreenShot = Image.open(path)
        width, height = openedScreenShot.size
        
        openedScreenShot.crop((250, 250, width - 50, height - 786 + 35 * self.Classes + 15)).save(path)

    def SendEmail(self, recipients, content):
        assert type(recipients) == list, "Expected a list of recipients got " + str(type(recipients))
        assert type(content) == dict, "Expected dictonary for content got" + str(type(content))

        self.Root.get("https://pupilpath.skedula.com/Mail/Show/?action=compose")

        self.Root.execute_script(
            f'arguments[0].setAttribute("value", "{",".join(recipients)}")', self.Root.find_element_by_id("recipientlist")
        )

        inputBoxes = [
            ('//*[@id="subject"]', content["Subject"]),
            ('//*[@id="reply"]/div[7]/div/p', content["Content"])
        ]

        for path, input in inputBoxes:
            waitToBeClickable("XPATH", path).send_keys(input)

        waitToBeClickable("XPATH", '//*[@id="reply"]/button').click()

        if self.Root.find_elements_by_xpath('//*[@id="success"]'):
            successMessage = self.Root.find_element_by_xpath('//*[@id="success"]')

            return True if successMessage.get_attribute("style").find("opacity") else False